#!/usr/bin/env python
# -*- coding: UTF-8 -*-

""" Script pour l'installation d'une distrib dans un serveur maitre """

import commands
import sys
import os
from creole import parsedico
import subprocess
import time
import glob
import re

DICO = parsedico.parse_dico()
ADRESSE_IP_DMZ = DICO['adresse_ip_dmz']
SERVEUR_MAJ = DICO['serveur_maj']
TYPE_MAJ = DICO['type_maj']
BRIDGE = DICO['nom_dmz']
DOMAINNAME = DICO['nom_domaine_local']
if DICO['adresse_ip_dmz']:
    ACTIVE_DMZ = 'oui'
else:
    ACTIVE_DMZ = 'non'

def clean_fstab(distmount):
    """
    nettoyage du fichier fstab
    """
    old_dist_fstab = run("grep -ni %s /etc/fstab |cut -d : -f 1" % distmount)
    if old_dist_fstab != '':
        line = int(old_dist_fstab) - 1
        run('sed -e %d,+1d /etc/fstab >/tmp/fstab' % line)
        run('cp /tmp/fstab /etc/fstab')
        run('rm /tmp/fstab')
 
def make_partitions(config, distmount, num, dist_device, taille):
    """
    Teste si une partition est montée si c'est le cas,
    récupère le device sur lequel est montée la partition
    démonte la partition nettoie le fichier /etc/fstab
    crée un volume physique
    crée le volume groupe
    teste la présence du VG eole
    teste si le VG eole est assez grand
    crée les Volumes Logiques
    """
    quota="no"
    partitions = config["PARTITIONS"]
    initmount = "/xen"
    montage = run("df -P | grep '%s' | awk '{ print $1 }'" % initmount)
    if montage != '':
        run("umount %s" % montage)
        clean_fstab(initmount)
        run("rm -rf %s" % initmount)
        ret = commands.getstatusoutput("pvdisplay %s >/dev/null" % montage)
        if not ret[0] == 0:
            run("pvcreate %s" % montage)
            run("vgcreate eole %s" % montage)
    clean_fstab(distmount)
    ret = commands.getstatusoutput("umount %s >/dev/null" % distmount)
    if ret[0] == 0 :
        # création du volume logique
        run("vgcreate eole %s" % dist_device)
        # création du volume physique
        run("pvcreate %s" % dist_device)
    # teste si le vg eole est assez grand
    #vg_size = run("vgdisplay eole 2>/dev/null| grep -iE 'Free *PE' | awk {'print $5'}")
    vg_size = run("vgdisplay eole -c 2> /dev/null|awk -F: '{ print $16*$13/1024 }'")
    vg_size = int(vg_size)
    if vg_size == '':
        sys.exit('\033[31mimpossible de detecter la partition des environnements virtuels !\033[0m')
    lv_total=0
    for logic_vol in partitions:
        lv_size = logic_vol[1]
        if lv_size == '*':
            if taille == '0':
                lv_total+=1024
            else:
                lv_total+=int(taille)*1024
        elif re.findall('G$',lv_size):
            lv_total+=int(lv_size[:-1])*1024
        elif re.findall('M$',lv_size):
            lv_total+=int(lv_size[:-1])
        else:
            sys.exit('\033[31mTaille lv_size inconnue\033[0m')
    if vg_size < lv_total :
        print "le volume group n'a pas assez d'espace disponible (%sM)" % vg_size
        print "Vous devez modifier la taille de vos partitions (minimum de %sM)" % lv_total
        sys.exit('\033[31mPlace insuffisante\033[0m')
    else:
        print "le volume group eole est suffissement grand %s PE" % vg_size
    if not os.path.isdir(distmount) :
        run("mkdir %s" % distmount)
    distmountroot = distmount.replace('private', 'root');
    run("echo '#!/bin/bash' > /etc/vz/conf/%s.mount" % num)
    run("echo '' > /etc/vz/conf/%s.umount" % num)
    run("modprobe dm-mod")
    for logic_vol in partitions :
        # le point de montage est la valeur
        lv_mount_point = logic_vol[0]
        option = ''
        optionfstab = 'defaults'
        if len(logic_vol) == 3:
             option = '-o %s' % logic_vol[2]
             optionfstab = logic_vol[2]
        name =  logic_vol[0].replace('/','')
        if name == "" :
            name = "root"
        lv_name = "%s-%s" % (num, name)
        lv_size = logic_vol[1]
        if lv_size == "*":
            if taille == '0':
                lv_size = run("vgdisplay eole -c 2> /dev/null|awk -F: '{ print $16*$13/1024 }'")
            else:
                lv_size = str(taille)+'G'
        ret = commands.getstatusoutput("lvdisplay /dev/eole/%s 2>/dev/null " % lv_name)
        if ret[0] != 0:
            print "création du volume Logique %s de taille %s pour le point de montage %s" % (lv_name, lv_size, lv_mount_point)
            run('lvcreate -n %s -L %s eole ' % (lv_name, lv_size))
        else :
            print "le volume logique %s existe" % lv_name
        if name != "swap" :
            print "formatage du volume Logique %s en EXT3" % lv_name
            run("mkfs.ext3 -q /dev/eole/%s 2>/dev/null " % lv_name)
            print "création du point de montage %s pour le Volume Logique %s" % (lv_mount_point, lv_name)
            if (lv_mount_point == '/'):
                if optionfstab != "defaults":
                    quota="yes"
                run('echo "/dev/eole/%s %s   auto    %s        0       2" >> /etc/fstab' % (lv_name, distmount, optionfstab))

            else:
                run('echo "mount /dev/eole/%s %s%s %s 2> /dev/null" >> /etc/vz/conf/%s.mount' % (lv_name, distmountroot, lv_mount_point, option, num))
                run('echo "umount /dev/eole/%s %s%s 2> /dev/null" > /etc/vz/conf/%s.umount.tmp'% (lv_name, distmountroot, lv_mount_point, num))
                run('cat /etc/vz/conf/%s.umount >> /etc/vz/conf/%s.umount.tmp' % (num, num))
                run('mv -f /etc/vz/conf/%s.umount.tmp /etc/vz/conf/%s.umount' % (num, num))
# pas de swap avec openvz 
        else :
            print "activation de l'espace de SWAP"
            run("mkswap /dev/eole/%s 2>/dev/null " % lv_name)
            run('echo "swapon /dev/eole/%s > /dev/null 2>&1" >> /etc/vz/conf/%s.mount' % (lv_name, num))
            run('echo "swapoff /dev/eole/%s > /dev/null 2>&1" >> /etc/vz/conf/%s.umount' % (lv_name, num))
    # montage de la partition scribe sur eclair-ng
    run("echo '#!/bin/bash' > /etc/vz/conf/%s.umount.tmp" % num)
    if re.findall('13$',num):
        num_scribe=str(num)[:-1]+'1'
        run('echo "[ -d /var/lib/vz/root/%s/home ] && mkdir -p /var/lib/vz/root/%s/partages && mount --bind /var/lib/vz/root/%s/home /var/lib/vz/root/%s/partages" >> /etc/vz/conf/%s.mount' % (num_scribe, num, num_scribe, num, num))
        run('echo "ret=\`mount|grep \\\"/var/lib/vz/root/%s/home on /var/lib/vz/root/%s/partages\\\"\`">> /etc/vz/conf/%s.umount.tmp' % (num_scribe, num, num))
        run('echo "[ ! \\\"\$ret\\\" = \\\"\\\" ] && umount /var/lib/vz/root/%s/partages" >> /etc/vz/conf/%s.umount.tmp' % (num, num))
    if re.findall('11$',num):
        num_scribe=str(num)[:-1]+'3'
        run('echo "[ ! \\\"\`mount|grep /var/lib/vz/root/%s/partages\`\\\" = \\\"\\\" ] && umount /var/lib/vz/root/%s/partages" >> /etc/vz/conf/%s.umount.tmp' % (num_scribe, num_scribe, num))
        run('echo "[ -d /var/lib/vz/root/%s/partages ] && mount --bind /var/lib/vz/root/%s/home /var/lib/vz/root/%s/partages" >> /etc/vz/conf/%s.mount' % (num_scribe, num, num_scribe, num))

    run('chmod +x /etc/vz/conf/%s.mount' % num)
    run('cat /etc/vz/conf/%s.umount >> /etc/vz/conf/%s.umount.tmp' % (num, num))
    run('mv -f /etc/vz/conf/%s.umount.tmp /etc/vz/conf/%s.umount' % (num, num))
    run('echo "exit 0" >> /etc/vz/conf/%s.umount' % num)
    run('chmod +x /etc/vz/conf/%s.umount' % num)
    print "montage des partitions"
    for logic_vol in partitions:
        lv_mount_point = logic_vol[0]
        name =  logic_vol[0].replace('/','')
        if name == "":
            lv_name = "%s-root" % num
            print "montage du volume logique root sur le point de montage %s" % lv_mount_point
            run("mkdir -p %s/%s" % (distmount, lv_mount_point))
            run("mount /dev/eole/%s %s/%s 2>/dev/null " % (lv_name, distmount, lv_mount_point))
    for logic_vol in partitions:
        lv_mount_point = logic_vol[0]
        name =  logic_vol[0].replace('/','')
        if name != "":
            lv_name = "%s-%s" % (num, name)
            if name != "swap":
                print "montage du volume logique %s sur le point de montage %s" % (lv_name, lv_mount_point)
                run("mkdir -p %s/%s" % (distmount, lv_mount_point))
                run("mount /dev/eole/%s %s/%s 2>/dev/null " % (lv_name, distmount, lv_mount_point))
    return quota

def remove_logical_volume(namelvm):
    if commands.getstatusoutput('/usr/sbin/vzlist -a -o veid,name | grep %s' % namelvm)[0] == 0 :
        print "attente de l'arrêt de %s ..." % namelvm
        run('/usr/sbin/vzctl stop %s' % namelvm)
        time.sleep(10)
    lv_to_remove = run("lvdisplay 2>/dev/null|grep 'LV Name' | grep %s- | awk '{print $3}'" % namelvm).split('\n')
    lv_to_umount = run("mount | grep /dev/mapper/eole-%s--root|awk '{ print $3 }'" % namelvm)
    if lv_to_umount != '':
        if os.path.isdir(lv_to_umount):
            run_subprocess_without_exit(['chroot', lv_to_umount, 'umount', '-a'])
            umount_partitions(namelvm, lv_to_umount.replace('private','root'))

    if commands.getstatusoutput("mount |grep /dev/mapper/eole-%s--root" % namelvm)[0] == 0:
        sys.exit("\033[31mveuillez redémarrer le serveur et relancer le script d'installation\033[0m")
    else:
        for lvs in lv_to_remove :
            run_subprocess(['lvremove', '-f',"%s" % lvs])

def check_cdrom_device():
    """testons si le device du cdrom est accessible"""
    real_cd_device = run("cat /proc/sys/dev/cdrom/info |grep \"drive name\" | awk '{ print $NF }'")
    cdrom_device = "/dev/"+real_cd_device
    cdrom_mount_point = run("grep cdrom /etc/fstab | awk {'print $2'}")
    if not os.path.isdir(cdrom_mount_point):
        run("mkdir %s" % cdrom_mount_point)
    return(cdrom_device, cdrom_mount_point)

def check_eole_cd():
    """fonction de détection du cdrom EOLENG
    création du point de montage dans le chroot et montage du cdrom"""

    cd_device, cd_mount_point = check_cdrom_device()
    print "Veuillez insérer le cdrom Eole NG dans le lecteur et validez"
    if raw_input() in ["ok", "ok", ""] :
        ret = commands.getstatusoutput("mount -o ro %s %s" % (cd_device, cd_mount_point))
        if ret[0] != 0 :
            print "Erreur lors du montage du CD"
            return(1)
        else :
            if os.path.isfile("%s/preseed/amonng.seed" % cd_mount_point):
                print "Le cdrom Eole a été trouvé."
                return(0)
            else :
                return(1)
    else :
        return(1)

def mount_chroot_cd(distmount):
    """montage du cdrom dans le chroot"""

    # création du point de montage pour le cdrom dans le chroot du VE
    run("mkdir %s/cdrom" % distmount)
    cdrom_mount_point = run("grep cdrom /etc/fstab | awk {'print $2'}")
    # montage du cdrom dans l'environnement chroot
    run("mount --bind %s %s/cdrom" % (cdrom_mount_point, distmount))

def chroot_dist(source_install, distrib, distmount, address):
    """chroot dans l'environnement du VE"""
    run("echo '127.0.0.1 localhost.localdomain localhost' > %s/etc/hosts" % distmount)
    run("echo '%s %s.%s %s' >> %s/etc/hosts" % (address, distrib, DOMAINNAME, distrib, distmount) )
    print "rentrée dans le chroot"
    run("cp /usr/share/eole/domu/scripts/prepare_domu.py %s/root/" % distmount)
    run("chmod a+x %s/root/prepare_domu.py" % distmount)
    cmd = ['chroot',  distmount, '/usr/bin/python', '/root/prepare_domu.py',source_install, distrib]
    run_subprocess(cmd)
    run("rm %s/root/prepare_domu.py" % distmount)

def umount_chroot_cd(distmount):
    """démontage du cdrom eole"""

    print "démontage du cdrom dans le chroot"
    run("umount %s/cdrom" % distmount)

def umount_cd():
    """démontage du cdrom"""

    print "démontage du cdrom"
    cdrom_mount_point = run("grep cdrom /etc/fstab | awk {'print $2'}")
    run("umount %s" % cdrom_mount_point)

def debootstrap_install(source, config, distmount):
    """
    fonction d'installation minimum
    soit depuis le réseau soit depuis le cdrom
    reseau : http://eoleng.ac-dijon.fr/ubuntu/
    cdrom : file:///media/cdrom0/
    """

    if source == "reseau" :
        source = "http://%s/ubuntu/" % SERVEUR_MAJ
    else :
        source = "file:///media/cdrom0/"

    infos = config["VERSIONS"]
    dist = ''
    for info in infos:
        if info[0] == 'distubuntu':
            dist=info[1]
    if dist == '':
        sys.exit('\033[31mIl faut un distubuntu dans le fichier de configuration\033[0m')
    print "récupération via debootstrap depuis %s" % source
    cmd = ['debootstrap', '--components=main,local,restricted,universe',
           '--include=ubuntu-keyring,ubuntu-minimal,openssh-server', '%s' % dist, distmount, '%s' % source]
    run_subprocess(cmd)

def generate_ssh_rsa_key(distmount):
    """génération de la clef rsa dans le VE pour les connexions SSH"""

    print "génération de la clef rsa dans le VE"
    if not os.path.isfile('/root/.ssh/id_rsa'):
        run_subprocess(['ssh-keygen', '-t', 'rsa', '-P', '', '-f', '/root/.ssh/id_rsa'])
    run('mkdir -p %s/root/.ssh/ 2>/dev/null' % distmount)
    run('cat /root/.ssh/id_rsa.pub > %s/root/.ssh/authorized_keys' % distmount)
    run('chmod 700 %s/root/.ssh/authorized_keys' % distmount)

def set_virt_timezone(distrib, distmount):
    """réglage de la timezone du VE"""

    print "reglage de la timezone du VE"
    run("cp /etc/timezone %s/etc/timezone" % distmount)
    run("cp /etc/localtime %s/etc/localtime" % distmount)

def set_virt_network_config(distrib, distmount, num, address, gw, netmask):
    """réglage de la configuration réseau"""

    print "configuration du réseau du VE"
#fixme: voir si vraiment pour eolevz !
    network_interfaces_file = ['auto lo', 'iface lo inet loopback', ' ',
        'auto eth0', 'iface eth0 inet static', 'address %s' % address,
        'netmask %s' % netmask,  'gateway %s' % gw]

    write_file('%s/etc/network/interfaces' % distmount, network_interfaces_file)
    resolv_conf= ['nameserver %s' % gw,'search %%DOMAINNAME']
    write_file('%s/etc/resolv.conf' % distmount, resolv_conf)
    if ACTIVE_DMZ != 'non':
        run('vzctl set %s --netif_add "eth0,00:E1:00:00:%s:%sA,veth%s,00:E1:00:00:%s:%sB" --save' % (num, num[0:2], num[2:3], num, num[0:2], num[2:3]))
    else:
        interface = run("ip route get %s|sed -ne '/ via /! s/^.* dev \([^ ]\+\) \+src \+.*$/\\1/p;Q'"% gw)
        if interface != '':
            run('echo "echo 100 > /proc/sys/net/ipv4/conf/%s/proxy_arp" >> /etc/vz/conf/%s.mount' % (interface, num))
        run('vzctl set %s --ipadd %s --save' % (num, address))
        run('echo "echo 1 > /proc/sys/net/ipv4/ip_forward" >> /etc/vz/conf/%s.mount' % num)
        run('cp /etc/resolv.conf %s/etc/resolv.conf' % distmount)
    run('echo "exit 0" >> /etc/vz/conf/%s.mount' % num)

def set_virt_sources_list(source_installation, config, distmount):
    print "Mise en place du sources.list pour %s" % source_installation
    infos = config["VERSIONS"]
    dist = ''
    eole = ''
    for info in infos:
        if info[0] == 'distubuntu':
            dist=info[1]
        if info[0] == 'disteole':
            eole=info[1]
    if dist == '':
        sys.exit('\033[31mIl faut un distubuntu dans le fichier de configuration\033[0m')
    if eole == '':
        sys.exit('\033[31mIl faut un disteole dans le fichier de configuration\033[0m')

    if source_installation == "cdrom":
        sources_list = "deb file:///cdrom %s main restricted universe local" % dist
        run('echo "%s" > %s/etc/apt/sources.list' % (sources_list, distmount))
    else:
        sources_list = ["deb http://%s/ubuntu/ %s main restricted universe" % ( SERVEUR_MAJ, dist),
                "deb http://%s/ubuntu/ %s-security main restricted universe" % ( SERVEUR_MAJ, dist ),
                "deb http://%s/eoleng/ eole-%s/\$(ARCH)/" % ( SERVEUR_MAJ, eole ),
                "deb http://%s/eoleng/ eole-%s/all/" % ( SERVEUR_MAJ, eole ),
                "deb http://%s/eoleng/ eole-%s-security/\$(ARCH)/" % ( SERVEUR_MAJ, eole ),
                "deb http://%s/eoleng/ eole-%s-security/all/" % ( SERVEUR_MAJ, eole )
                ]
        if TYPE_MAJ == 'complete':
            sources_list += ["deb http://%s/ubuntu/ %s-updates main restricted universe" % ( SERVEUR_MAJ, dist ),
                    "deb http://%s/eoleng/ eole-%s-updates/\$(ARCH)/" % ( SERVEUR_MAJ, eole ),
                    "deb http://%s/eoleng/ eole-%s-updates/all/" % ( SERVEUR_MAJ, eole )
                    ]

        for source in sources_list:
            if sources_list.index(source) == 0 :
                run('echo "%s" > %s/etc/apt/sources.list' % (source, distmount))
            else:
                run('echo "%s" >> %s/etc/apt/sources.list' % (source, distmount))

def umount_partitions(namelvm, mounted):
    """Démontage des partitions"""

    print "Démontage des partitions"
    partitions = run("df -P | grep '/dev/mapper/eole-%s' | awk '{ print $6 }' 2>/dev/null" % namelvm)
    partitions += '\n' + mounted
    liste_partitions = partitions.split("\n")
    liste_partitions.sort(reverse=True)
    for part in liste_partitions :
        run_subprocess_without_exit(['umount', part])
    partitions = run("swapon -s | grep '/dev/mapper/eole-%s' | awk '{ print $1 }' 2>/dev/null" % namelvm)
    if partitions != '':
        liste_partitions = partitions.split("\n")
        for part in liste_partitions :
            run_subprocess_without_exit(['swapoff', part])

def write_file(output_file, file_content):
    """ Ecrire file_content dans output_file """

    try:
        filehandle = file(output_file, 'w')
        filehandle.write('\n'.join(file_content))
        filehandle.close()
    except Exception:
        print "Impossible d'ecrire dans le fichier %s" % output_file

def run(cmd, noexit=False):
    """raccourci d'execution d'une commande shell"""
    ret, res = commands.getstatusoutput(cmd)
    if ret != 0 and noexit == False:
        print "\033[31mLa commande %s a echouee\033[0m" % cmd
        sys.exit(res)
    return res

def run_subprocess(cmd):
    """Raccourci d'execution pour l'execution d'une commande shell avec capture de STDOUT
    Attention, cmd est une liste :
    cmd='ls -al' s'ecrit cmd=['ls','-la']
    """
    returncode = run_subprocess_without_exit(cmd)
    if returncode:
        print "\033[31mLa commande %s a echoue\033[0m" % cmd
        sys.exit(returncode)

def run_subprocess_without_exit(cmd):
    process = subprocess.Popen(cmd, stdout=sys.stdout, stderr=sys.stderr, stdin=sys.stdin)
    process.communicate()
    return (process.returncode)

def read_config(conf_file):
    """Lire un fichier de configuration"""
    store={}
    try:
        input_file = open(conf_file,'r')
    except Exception:
        print "Impossible de lire le fichier %s" % conf_file

    raw_config = input_file.readlines()
    input_file.close()
    for lines in raw_config:
        line = lines.lstrip()
        if line:
            if line[0] == "[" :
                #Une section
                sec = line[1:len(line)-2].strip()
                store[sec]=[]
            elif line[0].strip() == '\n' or line[0].strip() == '#':
                #Ces lignes sont des commentaires
                continue
            else:
                if sec == 'FSTAB':
                    tab=line.strip()
                else:
                #Les clefs et les valeurs de la section en cours
                    tab=line.strip().replace(' ','',line.count(' ')).split('=')
                store[sec].append(tab)
    if 'PARTITIONS' not in store.keys():
        sys.exit("\033[31mLe fichier de configuration n'est pas valide\033[0m")
    else:
        return(store)

def remove_openvz (distrib, num):
    if os.path.isfile('/etc/vz/conf/%s.conf' % num): 
        print "Suppression du serveur %s" % distrib
        run('/usr/sbin/vzctl destroy %s' % num)
        run('rm -f /etc/vz/conf/%s.conf' % num)

def startdist(num, distmount):
    run_subprocess_without_exit(['/bin/mount', distmount])
    run_subprocess_without_exit(['/usr/sbin/vzctl', 'start', num])

def set_virt_configuration_openvz (config, distrib, num, quota, namevz):
    print 'Configuration openvz'
    run('vzctl set %s --applyconfig eole --save' % num)
    if distrib == 'eclair':
        run('vzctl set %s --devices c:10:229:rw --save' % num)
    run('vzctl set %s --name %s --save' % (num, namevz))
    if quota == "yes":
        space = run("df /var/lib/vz/private/%s|tail -n 1|awk '{ print $1 }'" % num)
        inode = run("df -i /var/lib/vz/private/%s|tail -n 1|awk '{ print $1 }'" % num)
        run('echo "DISK_QUOTA=\"yes\"" >> /etc/vz/conf/%s.conf' % num)
        run('vzctl set %s --diskspace %s:%s --save' % (num, space, space))
        run('vzctl set %s --diskinodes %s:%s --save' % (num, inode, inode))
        run('vzctl set %s --quotatime 0 --save' % num)
        run('vzctl set %s --quotaugidlimit 5000 --save' % num)
    else:
        run('echo "DISK_QUOTA=\"no\"" >> /etc/vz/conf/%s.conf' % num)
    run('echo "OSTEMPLATE=\"ubuntu-8.04\"" >> /etc/vz/conf/%s.conf' % num)

def create_dist (distrib, install_from_media, num, rang, namevz):
    print ''
    print 'Création de %s (serveur %s)' % (namevz, distrib) 
    distmount = "/var/lib/vz/private/%s" % num
    conffile = '/usr/share/eole/domu/conf/%s.conf' % distrib
    if not os.path.isfile(conffile):
        sys.exit('\033[31mAucun fichier de configuration pour %s \033[0m' % distrib)
    else:
        config = read_config(conffile)
        ret = commands.getstatusoutput("/usr/sbin/vzlist -o veid |grep %s" % num)
        if ret[0] != 0:
            address = DICO['ip_virt_%s' % distrib][rang-1]
            gw = DICO['gw_virt_%s' % distrib][rang-1]
            netmask = DICO['netmask_virt_%s' % distrib][rang-1]
            dist_device = "/dev/eole/%s-root" % num
            taille = DICO['place_virt_%s' % distrib][rang-1]

            quota = make_partitions(config, distmount, num, dist_device, taille)

            if install_from_media == "cdrom":
                mount_chroot_cd(distmount)
    
            debootstrap_install(install_from_media, config, distmount)

            set_virt_sources_list(install_from_media, config, distmount)
            chroot_dist(install_from_media, distrib, distmount, address)
            generate_ssh_rsa_key(distmount)
            set_virt_timezone(distrib, distmount)
            set_virt_configuration_openvz(config, distrib, num, quota, namevz)
            set_virt_network_config(distrib, distmount, num, address, gw, netmask)

            if install_from_media == 'cdrom':
                umount_chroot_cd(distmount)
            umount_partitions(num, distmount.replace('private','root'))
            run('/bin/hostname -F /etc/hostname')
            startdist(num, distmount)
        else:
            print 'Le serveur %s est déjà installé, ne fait rien' % distrib

def main():
    """Fonction main"""

    montage = run("uname -r|grep eolevz 2>/dev/null", True)
    if montage == '':
        print "\033[31mLe noyau n'est pas compatible avec OpenVZ\033[0m"
        print "\033[31mVous devez redémarrer et relancer l'instance\033[0m"
        return True
    ask_install_from_media = 'init'
    conffiles = [['eclair','13'], ['eolebase','14'], ['scribe','11'], ['horus','12']]
    for conffile in conffiles:
        distrib = conffile[0]
        iddistrib = conffile[1]
        if DICO['ip_virt_%s' % distrib] == ['']:
            numdist = 0
        else:
            numdist = len(DICO['ip_virt_%s' % distrib])
        list_lvm = run('lvdisplay 2> /dev/null|grep "LV Name"|grep -oE "/dev/eole/([[:digit:]]*%s|%s)-root"' % (iddistrib, distrib), True).split('\n')
        if list_lvm == ['']:
            numlist_lvm = 0
        else:
            numlist_lvm = len(list_lvm)
        if numdist > numlist_lvm:
            ask_install_from_media = 'yes'
        if not list_lvm == ['']:
            j = 1
            for i in list_lvm:
                j+=1
                num = re.findall('/dev/eole/(\w+)-root',i)[0]
                name = run('/usr/sbin/vzlist -a -H -n -o name %s 2> /dev/null' % num, True).strip().rstrip()
                if name == 'VE not found' or name == '-' or name == '':
                    name = num
                print ""
                print "Le serveur virtuel \"%s\" existe" % name
                print "Voulez-vous le supprimer (O/n) ?"
                print "attention, toutes les données seront perdues"
                if raw_input() in ["o", "O", ""]:
                    remove_logical_volume (num)
                    remove_openvz (distrib, num)
                    ask_install_from_media = 'yes'
                else:
                    if numdist+1 < j:
                        print 'Arrêt de %s' % num
                        run('/usr/sbin/vzctl stop %s' % num)
                        run('vzctl set %s --onboot no --save' %num)
                    else:
                        rep = run('/usr/sbin/vzlist -a -H -n %s -o status|awk \'{ print $1 }\'' % num, True)
                        if rep == 'stopped':
                            print 'Démarrage de %s' % num
                            run('/usr/sbin/vzctl start %s' % num)
                        run('vzctl set %s --onboot yes --save' %num)
                    if ask_install_from_media == 'init':
                        ask_install_from_media = 'no'
    if ask_install_from_media == 'yes' or ask_install_from_media == 'init':
        print ""
        print "Quelle méthode d'installation voulez-vous ?"
        rep = ''
        while rep not in ['reseau', 'cdrom']:
            rep = raw_input("Taper [reseau] pour installer depuis le réseau, taper [cdrom] pour installer depuis le cdrom :\n")
            if rep == 'cdrom':
                if check_eole_cd() != 0:
                    rep = 'none'
        install_from_media = rep

        if ACTIVE_DMZ != 'non':
            print ''
            print "Configuration du réseau"
            run('/sbin/ifdown %s' % BRIDGE)
            run('/sbin/ifup %s' % BRIDGE)
            run_subprocess_without_exit(['/etc/init.d/dhcp3-relay', 'restart'])
            if os.path.isfile('/etc/init.d/bind9'):
                run_subprocess_without_exit(['/etc/init.d/bind9', 'restart'])

        for conffile in conffiles:
            distrib = conffile[0]
            iddistrib = conffile[1]
            iplvm = DICO['ip_virt_%s' % distrib]
            if DICO['activate_virt_%s' % distrib] != 'oui':
                print ''
                print 'Le serveur %s virtuel est désactivé. Pour l\'activer, utilisez gen_config.' % distrib
                continue
            numdist = len(iplvm)+1
            for i in range(1, numdist):
                num = str(i) + conffile[1]
                namevz = DICO['name_virt_%s' % distrib][i-1]
                create_dist (distrib, install_from_media, num, i, namevz)

        if install_from_media == 'cdrom':
            umount_cd()

if __name__ == '__main__':
    main()

