#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import commands
import subprocess


def prepare_domu(source_installation, distrib):
    """
    Préparation du domU install a partir du cd ou du réseau 
    (cette fonction doit s'exécuter dans un chroot /xen-*)
    """
    
    print "Montage de proc, sys et dev/pts"
    run_subprocess(['mount', '-t' ,'proc', 'proc', '/proc'])
    run_subprocess(['mount', '-t' ,'sysfs', 'sys', '/sys'])
    run_subprocess(['mount', '-t' , 'devpts','""', '/dev/pts'])

    print "Désactivation des scripts start-stop-daemon et invoke-rc.d"
    services = ['/sbin/start-stop-daemon', '/usr/sbin/invoke-rc.d']
    for script in services :
        run("mv %s %s.REAL" % (script, script))
        fake_script = "#!/bin/sh\n\n echo 'Warning: Fake %s called doing nothing'\n" % script
        run("echo '%s' > %s" % (fake_script, script))
        run("chmod 755 %s" % script)

    print "Donner un nom au domU"
    run("echo %s > /etc/hostname" % distrib)
    run("echo %s > /etc/mailname" % distrib)
    run("hostname -F /etc/hostname")

    print "Configuration de debconf"
    debconf_config = ['#!/bin/bash', '. /usr/share/debconf/confmodule',
            'db_set debconf/frontend Noninteractive',
            'db_set debconf/priority critical']
    write_file('/tmp/debconf.sh', debconf_config)
    run("chmod a+x /tmp/debconf.sh")
    run_subprocess(['/tmp/debconf.sh'])

    print "Mise à jour de la liste des paquets"
    #run("apt-get update")
    run_subprocess(['apt-get', 'update'])
    cmd = ['apt-get', '-y', 'dist-upgrade', '--force-yes']
    run_subprocess(cmd)

    print "Installation pour la langue française"
    cmd = ['apt-get', '-y', 'install', 'language-pack-fr']
    run_subprocess(cmd)
    print "Génération des locales"
    run_subprocess(['/usr/sbin/locale-gen'])
    print "Configure les locales"
    run("echo LANG=\"fr_FR.UTF-8\" > /etc/default/locale")
    run("echo LANGUAGE=\"fr_FR.UTF-8\" >> /etc/default/locale")
    print "installation de eolebase-openvz"
    cmd = ['apt-get', '--force-yes', '-y', 'install', 'eolebase-openvz']
    run_subprocess(cmd)
    print "installation de %s" % distrib
    cmd = ['apt-get', '-y', 'install', '%s-ecole' % distrib, '--force-yes']
    run_subprocess(cmd)

    print "Replace les vrais scripts de démarrage"
    for script in services :
        run("mv %s.REAL %s" % (script, script))
    print "Désactive l'horloge matérielle"
    cmd = [ '/usr/sbin/update-rc.d', '-f', 'hwclock.sh', 'remove']
    run_subprocess(cmd)
    cmd = [ '/usr/sbin/update-rc.d', '-f', 'hwclockfirst.sh', 'remove']
    run_subprocess(cmd)
    print "Desactivation"
    cmd = [ '/usr/share/eole/postreconf/ecole']
    run_subprocess(cmd)
#    print "Désactivation de udev"
#    cmd = [ '/usr/sbin/update-rc.d', '-f', 'udev', 'remove']
#    run_subprocess(cmd)
#    cmd = [ '/usr/sbin/update-rc.d', '-f', 'udev-finish', 'remove']

#    print "Udev"
#    cmd = [ '/bin/rm', '-f', '/etc/udev/rules.d/85-hwclock.rules']
#    run_subprocess(cmd)
#    run('echo "#!/bin/bash" > /etc/rc.local')
#    run('echo "rm -rf /etc/udev/rules.d/70-persistent-net.rules" >> /etc/rc.local')
#    run('echo "exit 0" >> /etc/rc.local')

    print "Activation des mots de passe en shadow"
    run_subprocess(['/sbin/shadowconfig', 'on'])
#pouet
#    print "Suppression de tls"
#    run_subprocess(['mv', '/lib/tls', '/lib/tls.disabled'])
    print "Désactivation des tty"
    run("rm -f /etc/event.d/tty*")

    print "Remplacement /dev/console avec /dev/null"
    run("rm -f /dev/console")
    run("ln -s /dev/null /dev/console")

    print "Mets le mot de passe par défaut"
    run("echo 'root:eole' | chpasswd")

    print "Configuration les locales"
    run_subprocess(['locale-gen'])

    print "Fichier d'identification eolevz"
    run('touch /etc/sysconfig/eole/eolevz')

    print "Suppression des paquets en cache"
    run('apt-get clean')

    print "Démonte les systèmes de fichiers proc et sys"
    run("/bin/umount /dev/pts")
    run("/bin/umount /proc")
    run("/bin/umount /sys")
    run_subprocess_without_exit(['/bin/umount', '/*'])

def run(cmd):
    """raccourci d'execution d'une commande shell"""
    ret, res = commands.getstatusoutput(cmd)
    if ret != 0 :
        print "\033[31mLa commande %s a echouee\033[0m" % cmd
        sys.exit()
    return res

def run_subprocess(cmd):
    """Raccourci d'execution pour l'execution d'une commande shell avec capture de STDOUT
    Attention, cmd est une liste : 
    cmd='ls -al' s'ecrit cmd=['ls','-la']
    """
    process = subprocess.Popen(cmd, stdout=sys.stdout, stderr=sys.stderr, stdin=sys.stdin)
    process.communicate()
    if process.returncode:
        print "\033[31mLa commande %s a echoue\033[0m" % cmd
        subprocess.Popen(['umount', '/proc'], stdout=sys.stdout, stderr=sys.stderr, stdin=sys.stdin)
        subprocess.Popen(['umount', '/sys'], stdout=sys.stdout, stderr=sys.stderr, stdin=sys.stdin)
        subprocess.Popen(['umount', '/dev/pts'], stdout=sys.stdout, stderr=sys.stderr, stdin=sys.stdin)
        sys.exit(process.returncode)
#    process = subprocess.Popen(cmd,stdout=subprocess.PIPE)
#    while process.poll() is None:
#        print process.stdout.readline().strip()
#    if process.returncode != 0 :
#        print "La commande %s a échouée" % cmd
#        sys.exit()
#    else :
#        return(0)

def run_subprocess_without_exit(cmd):
    process = subprocess.Popen(cmd, stdout=sys.stdout, stderr=sys.stderr, stdin=sys.stdin)
    process.communicate()
    return (process.returncode)

def write_file(output_file, file_content):
    """ Ecrire file_content dans output_file """

    try:
        filehandle = file(output_file, 'w')
        filehandle.write('\n'.join(file_content))
        filehandle.close()
    except Exception:
         print "Impossible d'ecrire dans le fichier %s" % output_file

def main():
    """Foncion main"""

    source_install = sys.argv[1]
    distrib = sys.argv[2]
    prepare_domu(source_install, distrib)
    print "\033[32mOk\033[0m"


if __name__ == '__main__':
    main()
